import { Component } from '@angular/core';
import {
  CdkDragDrop,
  CdkDrag,
  CdkDropList,
  CdkDropListGroup,
  moveItemInArray,
  transferArrayItem,
} from '@angular/cdk/drag-drop';
import { Spell } from 'src/interfaces/spell';
import { DataService } from 'src/services/data/data.service';
import { ModalService } from 'src/services/modal/modal.service';
import { SpellModalComponent } from 'src/app/journal/spell-modal/spell-modal.component';

@Component({
  selector: 'app-journal-spellcards',
  templateUrl: './journal-spellcards.component.html',
  styleUrls: ['./journal-spellcards.component.scss'],
})
export class JournalSpellcardsComponent {
  public constructor(
    public dataAccessor: DataService,
    private modalAccessor: ModalService
  ) {
    this.loadSpells();
  }

  public level0: Spell[] = [];
  public level1: Spell[] = [];
  public level2: Spell[] = [];
  public level3: Spell[] = [];
  public level4: Spell[] = [];
  public level5: Spell[] = [];
  public level6: Spell[] = [];
  public level7: Spell[] = [];
  public level8: Spell[] = [];
  public level9: Spell[] = [];

  public listTranslator: any = {
    0: this.level0,
    1: this.level1,
    2: this.level2,
    3: this.level3,
    4: this.level4,
    5: this.level5,
    6: this.level6,
    7: this.level7,
    8: this.level8,
    9: this.level9,
  };

  public showSpellList: boolean[] = [
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
  ];

  ///////// FUNCTIONS //////////

  public openModal(
    isUpdate: boolean,
    level: number,
    spellIndex?: number
  ): void {
    console.log(spellIndex);
    this.modalAccessor.openModal(SpellModalComponent, {
      spell:
        spellIndex !== undefined
          ? JSON.parse(JSON.stringify(this.listTranslator[level][spellIndex]))
          : undefined,
      isUpdate: isUpdate,
      level: level,
    });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        if (result.state === 'update') {
          console.log(result.data);
          this.updateSpell(result.data, level!, spellIndex!);
        } else if (result.state === 'add') {
          console.log('im else: ', result.data);
          this.addSpell(result.data, level);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public addSpell(spell: Spell, level: number) {
    console.log('The level: ', level);
    console.log('The levellist:', this.listTranslator[level]);
    console.log('The new spell: ', spell);
    this.listTranslator[level].push(spell);
    console.log(this.listTranslator[level]);
    // this.updateSpellsInDatabase();
  }

  public updateSpell(spell: Spell, level: number, index: number): void {
    this.listTranslator[level][index] = spell;
    // this.updateSpellsInDatabase();
  }

  toggleSpellList(index: number) {
    console.log(index);
    this.showSpellList[index] = !this.showSpellList[index];
  }

  drop(event: CdkDragDrop<any[]>) {
    if (event.previousContainer === event.container) {
      moveItemInArray(
        event.container.data,
        event.previousIndex,
        event.currentIndex
      );
      console.log(event.container.data);
    } else {
      transferArrayItem(
        event.previousContainer.data,
        event.container.data,
        event.previousIndex,
        event.currentIndex
      );

      const movedSpell = event.container.data[event.currentIndex];
      const newContainer = event.container.id;

      switch (newContainer) {
        case 'cdk-drop-list-0':
          movedSpell.level = 0;
          break;
        case 'cdk-drop-list-1':
          movedSpell.level = 1;
          break;
        case 'cdk-drop-list-2':
          movedSpell.level = 2;
          break;
        case 'cdk-drop-list-3':
          movedSpell.level = 3;
          break;
        case 'cdk-drop-list-4':
          movedSpell.level = 4;
          break;
        case 'cdk-drop-list-5':
          movedSpell.level = 5;
          break;
        case 'cdk-drop-list-6':
          movedSpell.level = 6;
          break;
        case 'cdk-drop-list-7':
          movedSpell.level = 7;
          break;
        case 'cdk-drop-list-8':
          movedSpell.level = 8;
          break;
        case 'cdk-drop-list-9':
          movedSpell.level = 9;
          break;
      }
    }
  }

  private loadSpells(): void {
    this.level0 = this.dataAccessor.spellLevel0;
    this.level1 = this.dataAccessor.spellLevel1;
    this.level2 = this.dataAccessor.spellLevel2;
    // this.level3 = this.dataAccessor.spellLevel3;
    // this.level4 = this.dataAccessor.spellLevel4;
    // this.level5 = this.dataAccessor.spellLevel5;
    // this.level6 = this.dataAccessor.spellLevel6;
    // this.level7 = this.dataAccessor.spellLevel7;
    // this.level8 = this.dataAccessor.spellLevel8;
    // this.level9 = this.dataAccessor.spellLevel9;
  }
}
